package pcksniff;

/**
* This is the main class of the Sniffer package. This class provides all the methods necessary
* to capture and filter packets received from a network device.
* 
*/

public class Sniffer
{ 
  private Sniffer()
  {
  }	
	
  static 
  { System.loadLibrary("Sniffer");
  }
  
  /**
  * The adapter normal mode.
  */ 	
  public static final int NORMAL_MODE = 0 ;
  
  /**
  * The adapter promiscuous mode.
  */
  public static final int PROMISCUOUS_MODE = 1 ; 
  
  /**
  * Optimization compiling a bpf program off.
  */
  public static final int OPTIMIZ_OFF = 0 ;
  
  /**
  * Optimization compiling a bpf program on.
  */
  public static final int OPTIMIZ_ON = 1 ;
  	
  private static native byte[] pcapOpenLive(String device,int snaplen,int promisc,int to_ms,StringBuffer ebuf) ; 
  private static native byte[] pcapOpenOffline(String fname,StringBuffer ebuf) ;
  private static native byte[] pcapDumpOpen(byte[] pcap, String fname) ; 
  private static native String pcapLookupDev(StringBuffer ebuf) ;  
  private static native int    pcapLookupNetNumber(String device,StringBuffer ebuf) ;
  private static native int    pcapLookupNetMask(String device,StringBuffer ebuf) ;
  private static native int    pcapDispatch(byte[] pcap,PacketConsumer consumer,int cnt) ;
  private static native int    pcapLoop(byte[] pcap,PacketConsumer consumer,int cnt) ;  
  private static native int    pcapDispatchDump(byte[] pcap,byte[] dumper,int cnt) ;
  private static native int    pcapLoopDump(byte[] pcap,byte[] dumper,int cnt) ;
  private static native byte[] pcapAllocateBpfProgram() ;
  private static native int    pcapCompile(byte[] pcap,byte[] bpf_program,String query,int optmize,int netMask) ;  
  private static native int    pcapSetFilter(byte[] pcap,byte[] bpf_program) ;
  private static native int    pcapStatsPacketsReceived(byte[] pcap) ;
  private static native int    pcapStatsPacketsDropped(byte[] pcap) ;
  private static native void   pcapClose(byte[] pcap) ;
  private static native void   pcapDumpClose(byte[] dmp) ;

  /**
  * <BR>
  * Is used to obtain an Adapter class that identifies the network device.
  * <BR>
  * <BR>
  * @param device  specifies the name of the network device to open.
  * @param snaplen specifies the maximum number of bytes to capture.   
  * @param promisc specifies the normal or the promiscuos mode. 
  * @param to_ms   specifies the read timeout in milliseconds.
  * @param ebuf    if an error occurs the ebuf.append(String) method is called passing the error message.
  * 
  * @return a new Adapter object. 
  * 
  * <BR>
  * <BR>
  * Call the isNull() method of the Adapter class to check if the openLive succeed.
  * <BR>
  * <BR>
  * Example:<BR>
  * <BR>
  * <B>
  * StringBuffer ebuf = new StringBuffer() ;<BR>
  * Adapter adapter = Sniffer.openLive("N1001",1024,PROMISCUOUS_MODE,10,ebuf) ;<BR>
  * <BR>       
  * if ( adapter.isNull() )<BR> 
  * { System.out.println(ebuf) ;<BR>
  * &nbsp;&nbsp;System.exit(-1) ;<BR>
  * }<BR>
  * else<BR>
  * { System.out.println("Adapter Opened") ;<BR>     
  * }<BR>
  * </B>   
  *
  * 
  */
  
  public static Adapter openLive(String device, int snaplen, int promisc, int to_ms, StringBuffer ebuf) 
  {  byte[] pointer =  pcapOpenLive(device,snaplen,promisc,to_ms,ebuf) ; 	
     return new Adapter(pointer) ;  	
  } 
    
  /**
  * <BR>
  * Opens a captured saved file for reading. 
  * <BR>
  * <BR>
  * @param fname   specifies the name of the file to open. 
  * @param ebuf    if an error occurs the ebuf.append(String) method is called passing the error message.
  * 
  * @return a new Adapter object. 
  * 
  * <BR>
  * <BR>
  * See openLive for more details.
  * 
  */
   
  public static Adapter openOffline(String fname,StringBuffer ebuf) 
  {  byte[] pointer =  pcapOpenOffline(fname,ebuf) ; 	
     return new Adapter(pointer) ;  	
  } 
  
  /**
  * <BR>
  * Opens a file for writing packets. Returns a Dumper object suitable for use with dispatchDump() and
  * loopDump().  
  * <BR>
  * <BR>
  * @param adapter is an Adapter returned by openLive.
  * @param fname specifies the name of the file to open. 
  * 
  * @return a new Dumper object. 
  * 
  * 
  */
      
  public static Dumper dumpOpen(Adapter adapter,String fname )
  {  byte[] pointer =  pcapDumpOpen(adapter.getPointer(),fname) ; 	
     return new Dumper(pointer) ;  		
  }
        
  /**
  * <BR>
  * Returns a device name suitable for use with openLive(). 
  * <BR>
  * <BR>
  * @param ebuf if an error occurs the ebuf.append(String) method is called passing the error message.
  * 
  * @return a network device name 
  * 
  */ 
   
  public static String lookupDev(StringBuffer ebuf)
  {  return pcapLookupDev(ebuf) ;
  }    

  /**
  * <BR>
  * Returns the network number associated with a network device. 
  * <BR>
  * <BR>
  * @param ebuf if an error occurs the ebuf.append(String) method is called passing the error message.
  * 
  * @return the network number  
  * 
  */
  
  public static int lookupNetNumber(String device,StringBuffer ebuf)
  {  int netNumber = pcapLookupNetNumber(device,ebuf) ;
     return netNumber ;
  }
  
  /**
  * <BR>
  * Returns the network mask associated with a network device. 
  * <BR>
  * <BR>
  * @param ebuf if an error occurs the ebuf.append(String) method is called passing the error message.
  * 
  * @return the network mask
  * 
  */
  
  public static int lookupNetMask(String device,StringBuffer ebuf)
  {  int netMask = pcapLookupNetMask(device,ebuf) ;
     return netMask  ;
  }
    
  /**
  * <BR>
  * Collects and process the packets. For each packet received the method <I>consume</I> of
  * the PacketPacketConsumer class is called.
  * <BR>
  * <BR>
  * @param adapter is an Adapter returned by <I>openLive</I> or <I>openOffline</I> 
  * @param consumer is a class that implements the PacketPacketConsumer interface
  * @param cnt specifies the maximum number of packets to process before returning 
  *
  * @return -1 if an error occured
  * 
  */
    
  public static int dispatch(Adapter adapter,PacketConsumer consumer,int cnt)
  { return pcapDispatch(adapter.getPointer(),consumer,cnt) ;
  }
  
  /**
  * <BR>
  * Is similar to dispatch except that instead of returning, it keeps reading packets
  * until cnt packets are processed or an error occurs. A negative number of cnt causes 
  * an infinite loop.  
  * <BR>
  * <BR>
  * @param adapter is an Adapter returned by <I>openLive</I> or <I>openOffline</I> 
  * @param consumer is a class that implements the PacketPacketConsumer interface
  * @param cnt specifies the maximum number of packets to process before returning 
  *
  * @return -1 if an error occured
  * 
  */
  
  public static int loop(Adapter adapter,PacketConsumer consumer,int cnt)
  { return pcapLoop(adapter.getPointer(),consumer,cnt) ;
  }
  
  /**
  * <BR>
  * Collects and Saves the packets received into the file opened with dumpOpen. 
  * <BR>
  * <BR>
  * @param adapter is an Adapter returned by <I>openLive</I> or <I>openOffline</I>.
  * @param dumper is a Dumper returned by openDumper.
  * @param cnt specifies the maximum number of packets to process before returning. 
  *
  * @return -1 if an error occured.
  * 
  */
  
  public static int dispatchDump(Adapter adapter,Dumper dumper,int cnt)
  { return pcapDispatchDump(adapter.getPointer(),dumper.getPointer(),cnt ) ;
  }
    
  /**
  * <BR>
  * Is similar to dispatchDump except that instead of returning, it keeps reading packets
  * until cnt packets are processed or an error occurs. A negative number of cnt causes 
  * an infinite loop.  
  * <BR>
  * <BR>
  * @param adapter is an Adapter returned by <I>openLive</I> or <I>openOffline</I>.
  * @param dumper is a Dumper returned by openDumper.
  * @param cnt specifies the maximum number of packets to process before returning. 
  *
  * @return -1 if an error occured.
  * 
  */
  
  public static int loopDump(Adapter adapter,Dumper dumper,int cnt)
  { return pcapLoopDump(adapter.getPointer(),dumper.getPointer(), cnt) ;
  }
    
  /**
  * <BR>
  * Creates a BpfProgram object to use with compile method.
  * <BR>
  * <BR>
  *
  * @return a new BfpProgram class.
  * 
  */
  
  public static BpfProgram allocateBpfProgram() 
  { byte[] pointer = pcapAllocateBpfProgram() ;
    return new BpfProgram(pointer) ;   	
  }
  
  /**
  * <BR>
  * Compiles a string into a filter program.
  * <BR>
  * <BR>
  * @param adapter is an Adapter returned by <I>openLive</I> or <I>openOffline</I>.
  * @param bpf_program is a bpfProgram object returned by allocateBpfProgram.
  * @param prg is the program to compile.
  * @param optimize controls whether optimization on the resulting code is performed.
  * @netmask specifies the netmask of the local net.
  * 
  * @return -1 if an error occured.
  * 
  */
    
  public static int compile(Adapter adapter,BpfProgram bpf_program,String prg,int optmize,int netMask)
  { return pcapCompile(adapter.getPointer(),bpf_program.getPointer(),prg,optmize,netMask) ;  
  }  
  
  /**
  * <BR>
  * Specifies a filter program to use for filtering packets.
  * <BR>
  * <BR>
  * @param adapter is an Adapter returned by <I>openLive</I> or <I>openOffline</I>.
  * @param bpf_program is a bpfProgram object returned by allocateBpfProgram and compiled by compile.
  * 
  * @return -1 if an error occured.
  * 
  */
      
  public static int setFilter(Adapter adapter,BpfProgram bpf_program)
  { return pcapSetFilter(adapter.getPointer(),bpf_program.getPointer()) ;
  }  

  /**
  * <BR>
  * Returns the number of packets received from the start of the run to the time of the call. 
  * <BR>
  * <BR>
  * @param adapter is an Adapter returned by <I>openLive</I> or <I>openOffline</I>.
  * 
  * @return -1 if an error occured.
  * 
  */
   
  public static int statsPacketsReceived(Adapter adapter) 
  { return pcapStatsPacketsReceived(adapter.getPointer()) ;
  }
  
  /**
  * <BR>
  * Returns the number of packets dropped from the start of the run to the time of the call. 
  * <BR>
  * <BR>
  * @param adapter is an Adapter returned by <I>openLive</I> or <I>openOffline</I>.
  * 
  * @return -1 if an error occured.
  * 
  */
  
  public static int statsPacketsDropped(Adapter adapter) 
  { return pcapStatsPacketsDropped(adapter.getPointer()) ;
  } 
  
  /**
  * <BR>
  * Closes an Adapter and deallocates the resources. 
  * <BR>
  * <BR>
  * @param adapter is an Adapter previously opened by <I>openLive</I> or <I>openOffline</I>.
  * 
  */
  
  public static void close(Adapter adapter)
  { pcapClose(adapter.getPointer()) ;
  }
  
  /**
  * <BR>
  * Closes a Dumper and the associated 'save file'. 
  * <BR>
  * <BR>
  * @param dumper is a Dumper previously opened by <I>openDumper</I>.
  * 
  */
  
  public static void dumpClose(Dumper dumper) 
  { pcapDumpClose(dumper.getPointer()) ;
  }
    
}